% Copyright 1994-2001 The MathWorks, Inc. 
% $Revision: 1.8 $

echo on

% Matrices of the linearized paper machine model

A=[-1.93 0 0 0; .394 -.426 0 0; 0 0 -.63 0; .82 -.784 .413 -.426];
B=[1.274 1.274 0 0;0 0 0 0;1.34 -.65 .203 .406;0 0 0 0];
C=[0 1 0 0; 0 0 1 0; 0 0 0 1];
D=zeros(3,4);

% Discretize the linear model and save in MOD form.
% Then calculate & plot step responses.

dt=2;

[PHI,GAM]=c2dmp(A,B,dt);
minfo=[dt,4,2,1,1,3,0];
imod=ss2mod(PHI,GAM,C,D,minfo);
plotstep(mod2step(imod,30));

% Define controller parameters

P=10;              % Prediction horizon
M=3;               % Control horizon
ywt=[1,0,1];       % Equal weighting of y1 and y3, no control of y2
uwt=0.6*[1 1];     % Equal weighting of u1 and u2.
ulim=[-10*[1 1] 10*[1 1] 2*[1 1]];      % Constraints on u
ylim=[];                                % No constraints on y
Kest=[];                                % Default estimator

% Simulation using SCMPC -- no model error

pmod=imod;            % plant and internal model are identical
setpts=[1 0 0];       % servo response to step in y1 setpoint
tend=30;              % duration of simulation
z=[];                 % measurement noise
v=0;                  % measured disturbance
d=0;                  % unmeasured disturbance

[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Repeat for step in setpoint of y3.

setpts=[0 0 1];       % servo response to step in y3 setpoint
[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Change the output weighting to allow better control of y3
% at the expense of y1.

ywt=[0.2 0 1];        % Unequal output weighting
setpts=[1 0 0];       % servo response to step in y1 setpoint
[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Test the new design for a step in setpoint of y3.

ywt=[0.2 0 1];        % Unequal output weighting
setpts=[0 0 1];       % servo response to step in y3 setpoint
[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Test the design for a step in the measured disturbance.

setpts=[0 0 0];       % Hold outputs at zero
v=1;                  % unit step in measured disturbance
[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Test the design for a step in the unmeasured disturbance.

v=0;                  % Measured disturbance is now zero
d=1;                  % Unit step in unmeasured disturbance
[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Design an alternative to the default state estimator.
% This one is the optimal Kalman estimator for the modeled
% disturbance.

Q=30;
R=1*diag([1 1 1]);
Kest=smpcest(imod,Q,R);

[y,u,ym]=scmpc(pmod,imod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause


% Try another alternative estimator.
% This one is based on the output-disturbance model.

taus=[5 5 5];                % Time constants of output disturbances
signoise=[10 10 10];         % Signal-to-noise for each output
[Kest,newmod]=smpcest(imod,taus,signoise);

[y,u]=scmpc(pmod,newmod,ywt,uwt,M,P,tend, ...
               setpts,ulim,ylim,Kest,z,v,d);
plotall(y,u,dt)
disp('Hit a key to continue'), pause

