
% Matrices of the linearized paper machine model

% Copyright 1994-2001 The MathWorks, Inc. 
% $Revision: 1.9 $

A=[-1.93 0 0 0; .394 -.426 0 0; 0 0 -.63 0; .82 -.784 .413 -.426];
B=[1.274 1.274 0 0;0 0 0 0;1.34 -.65 .203 .406;0 0 0 0];
C=[0 1 0 0; 0 0 1 0; 0 0 0 1];
D=zeros(3,4);

% Discretize the linear model and save in MOD form.

dt=2;
[PHI,GAM]=c2dmp(A,B,dt);
minfo=[dt,4,2,1,1,3,0];
imod=ss2mod(PHI,GAM,C,D,minfo);

% Define controller parameters.  The controller design is the same
% as for the linear case.

P=10;              % Prediction horizon
M=3;               % Control horizon
ywt=[0.2,0,1];     % unequal weighting of y1 and y3, no control of y2
uwt=0.6*[1 1];      % Equal weighting of u1 and u2.
ulim=[-10*[1 1] 10*[1 1] 2*[1 1]];      % Constraints on u
ylim=[];                                % No constraints on y

% Estimator design

Q=30;
R=1*diag([1 1 1]);
Kest=smpcest(imod,Q,R);

% Simulation using SCMPCNL.  We first try a step in the setpoint of y1.
pmod='papmach';       % plant is nonlinear ODE model
%pmod='paprmach';       % plant is nonlinear ODE model -- Simulink version
setpts=[1 0 0];       % servo response to step in y1 setpoint
tend=30;              % duration of simulation
z=[];                 % measurement noise
v=0;                  % measured disturbance is zero initially
d=0;                  % unmeasured disturbance is zero initially
du=[];                % No unmeasured disturbance at inputs;
x0=zeros(4,1);        % initial state of nonlinear plant
u0=zeros(3,1);        % initial manipulated variables & measured disturbance
stepsize=0.05;        % for numerical integration.

% Ignore the warning messages in the following simulations.
% They are warning that SCMPCNL has made a decision that might
% not be correct in general, but is for this case.

[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
                        setpts,ulim,ylim,Kest,z,v, ...
                        d,du,x0,u0,stepsize);
plotall(y,u,dt)
disp('Hit any key to continue'), pause

% You can verify that that this is essentially the same as for the
% linear case.  The same is true for a change in the y3 setpoint,
% and the responses to the measured and unmeasured disturbances
% we tested in the linear plant demo.
% To verify this, remove "%" characters from the start of the
% following lines.  (These are being skipped here to save time...).

%setpts=[0 0 1];       % servo response to step in y3 setpoint
%[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
%                        setpts,ulim,ylim,Kest,z,v, ...
%                        d,du,x0,u0,stepsize);
%plotall(y,u,dt)
%pause
%setpts=[0 0 0];       % hold outputs at zero
%v=1;                  % unit step in measured disturbance
%[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
%                        setpts,ulim,ylim,Kest,z,v, ...
%                        d,du,x0,u0,stepsize);
%plotall(y,u,dt)
%pause


% To make the nonlinear effects more obvious, we try a
% step in the unmeasured disturbance of magnitude 7 times
% larger than that used in the linear simulation.

setpts=[0 0 0];       % hold outputs at zero
v=0;                  % no measured disturbance
d=7;                  % larger step in unmeasured disturbance
[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
                        setpts,ulim,ylim,Kest,z,v, ...
                        d,du,x0,u0,stepsize);
plotall(y,u,dt)
disp('Hit any key to continue'), pause

% The response is clearly different from the linear case.
% Now we try a slightly larger disturbance.

d=8;                  % even larger step in unmeasured disturbance
[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
                        setpts,ulim,ylim,Kest,z,v, ...
                        d,du,x0,u0,stepsize);
plotall(y,u,dt)
disp('Hit any key to continue'), pause

% Control has degraded dramatically.  Reasonable performance can
% be restored by "de-tuning" the controller.  The conventional
% way is to increase the penalty on input variable moves.  To see
% this (lengthy) simulation, remove "%" from the beginning of
% the following lines:

%uwt=10*[1 1];         % De-tune controller by increasing penalty on
                       % manipulated variable moves.
%tend=100;             % Increase simulation time.
%[y,u,ym,x]=scmpcnl(pmod,imod,ywt,uwt,M,P,tend, ...
%                        setpts,ulim,ylim,Kest,z,v, ...
%                        d,du,x0,u0,stepsize);
%plotall(y,u,dt)
%disp('Hit any key to continue'), pause


% Now we see whether disturbance rejection could be improved by
% a reduction in the sampling period.  We also test the output
% disturbance estimator design (which is the same as in the linear
% case.

dt=0.25;                           % Change sampling period
[PHI,GAM]=c2dmp(A,B,dt);           % Get new discrete-time linear model
minfo=[dt,4,2,1,1,3,0];            % Replace old model
imod=ss2mod(PHI,GAM,C,D,minfo);
uwt=0.6*[1 1];                     % Restore original weighting of u1 and u2.
taus=[5 5 5];                % Time constants of output disturbances
signoise=[10 10 10];         % Signal-to-noise for each output
[Kest,newmod]=smpcest(imod,taus,signoise);
d=1;                               % Return to original disturbance magnitude
[y,u,ym,x]=scmpcnl(pmod,newmod,ywt,uwt,M,P,tend, ...
                        setpts,ulim,ylim,Kest,z,v, ...
                        d,du,x0,u0,stepsize);
plotall(y,u,dt)

% Disturbance rejection has improved significantly.
